@@ -1,4 +1,4 @@
-/* 12jan15abu
+/* 08feb17abu
  * (c) Software Lab. Alexander Burger
  */
 
@@ -33,8 +33,10 @@ function formKey(event) {
 
 function fldChg(field) {
    Chg = true;
-   if (!InBtn && Key != 13)
+   if (!InBtn && (Key != 13 || field.type == "textarea")) {
       post(field.form, false, null, null);
+      Key = 0;
+   }
    return true;
 }
 
@@ -84,6 +86,14 @@ function setHref(fld, url) {
    fld.href = decodeURIComponent(url);
 }
 
+function idFocus(fld) {
+   try {
+      document.createEvent("TouchEvent");
+      return true;
+   } catch (e) {}
+   setTimeout(function() {document.getElementById(fld).focus()}, 420);
+}
+
 /*** Form submit ***/
 function doPost(form) {
    for (var i = 0; ; ++i) {
