@@ -1,4 +1,4 @@
-/* 12apr17abu
+/* 10nov17abu
  * (c) Software Lab. Alexander Burger
  */
 
@@ -31,12 +31,13 @@ typedef struct name {
    int port;
    uid_t uid;
    gid_t gid;
-   char *dir, *log, *ev[4], *av[1];
+   char *dir, *log, *ev[5], *av[1];
 } name;
 
 static bool Hup;
 static name *Names;
 static char *Config;
+static int CliSock, SrvSock;
 static char Ciphers[] = "ECDHE-RSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-SHA256:ECDHE-RSA-AES128-SHA:AES128-GCM-SHA256:AES128-SHA256:AES128-SHA:DES-CBC3-SHA";
 
 static char Head_302[] =
@@ -61,7 +62,7 @@ static int readNames(void) {
    name *np, **t;
    int port, cnt;
    struct passwd *pw;
-   char *p, *ps, line[4096];
+   char *p, *q, *ps, line[4096];
    static char delim[] = " \n";
 
    if (!(fp = fopen(Config, "r")))     // Lines ordered by
@@ -90,7 +91,9 @@ static int readNames(void) {
             strcpy(p, "HOME="), strcpy(p+5, pw->pw_dir);
             p = np->ev[2] = malloc(5 + strlen(ps) + 1);
             strcpy(p, "PORT="), strcpy(p+5, ps);
-            np->ev[3] = NULL;
+            p = np->ev[3] = malloc(5 + strlen(q = getenv("LANG")) + 1);
+            strcpy(p, "LANG="), strcpy(p+5, q);
+            np->ev[4] = NULL;
             np->dir = strdup(strtok(NULL, delim));
             np->log = *(p = strtok(NULL, delim)) == '^'? NULL : strdup(p);
             cnt = 0;
@@ -128,7 +131,7 @@ static void freeNames(name *np) {
    free(np->dir);
    if (np->port) {
       free(np->log);
-      for (i = 0; i < 3; ++i)
+      for (i = 0; i < 4; ++i)
          free(np->ev[i]);
       for (i = 0; np->av[i]; ++i)
          free(np->av[i]);
@@ -311,6 +314,8 @@ static pid_t Buddy;
 
 static void doSigAlarm(int n __attribute__((unused))) {
    kill(Buddy, SIGTERM);
+   shutdown(CliSock, SHUT_RDWR);
+   shutdown(SrvSock, SHUT_RDWR);
    exit(0);
 }
 
@@ -332,7 +337,7 @@ static void iSignal(int n, void (*foo)(int)) {
 }
 
 int main(int ac, char *av[]) {
-   int cnt = ac>4? ac-3 : 1, ports[cnt], n, sd, cli, srv;
+   int cnt = ac>4? ac-3 : 1, ports[cnt], n, sd;
    struct sockaddr_in6 addr;
    char s[INET6_ADDRSTRLEN];
    char *p, *q, *gate;
@@ -384,7 +389,7 @@ int main(int ac, char *av[]) {
          ports[0] = readNames();
       }
       socklen_t len = sizeof(addr);
-      if ((cli = accept(sd, (struct sockaddr*)&addr, &len)) >= 0 && (n = fork()) >= 0) {
+      if ((CliSock = accept(sd, (struct sockaddr*)&addr, &len)) >= 0 && (n = fork()) >= 0) {
          if (!n) {
             name *np;
             int fd, port, i;
@@ -394,11 +399,11 @@ int main(int ac, char *av[]) {
 
             alarm(420);
             if (ssl) {
-               SSL_set_fd(ssl, cli);
+               SSL_set_fd(ssl, CliSock);
                if (SSL_accept(ssl) < 0)
                   return 1;
             }
-            n = rdLine(ssl, cli, buf, sizeof(buf));
+            n = rdLine(ssl, CliSock, buf, sizeof(buf));
             alarm(0);
             if (n < 6)
                return 1;
@@ -439,10 +444,10 @@ int main(int ac, char *av[]) {
                if (ssl)
                   sslWrite(ssl, buf, i);
                else
-                  wrBytes(cli, buf, i);
+                  wrBytes(CliSock, buf, i);
                return 0;
             }
-            if ((srv = gateConnect(port, np)) < 0) {
+            if ((SrvSock = gateConnect(port, np)) < 0) {
                if (!memchr(q,'~', buf + n - q))
                   return 1;
                if ((fd = open("void", O_RDONLY)) < 0)
@@ -451,64 +456,64 @@ int main(int ac, char *av[]) {
                if (ssl)
                   sslWrite(ssl, Head_410, strlen(Head_410));
                else
-                  wrBytes(cli, Head_410, strlen(Head_410));
+                  wrBytes(CliSock, Head_410, strlen(Head_410));
                alarm(0);
                while ((n = read(fd, buf, sizeof(buf))) > 0) {
                   alarm(420);
                   if (ssl)
                      sslWrite(ssl, buf, n);
                   else
-                     wrBytes(cli, buf, n);
+                     wrBytes(CliSock, buf, n);
                   alarm(0);
                }
                return 0;
             }
 
-            wrBytes(srv, buf, p - buf);
+            wrBytes(SrvSock, buf, p - buf);
             if (*q == '/')
                ++q;
             p = q;
             while (*p++ != '\n')
                if (p >= buf + n)
                   return 1;
-            wrBytes(srv, q, p - q);
+            wrBytes(SrvSock, q, p - q);
             inet_ntop(AF_INET6, &addr.sin6_addr, s, INET6_ADDRSTRLEN);
-            wrBytes(srv, buf2, sprintf(buf2, gate, s));
+            wrBytes(SrvSock, buf2, sprintf(buf2, gate, s));
             if (ssl)
-               wrBytes(srv, buf2, sprintf(buf2, "X-Pil: *Cipher=%s\r\n", SSL_get_cipher(ssl)));
-            wrBytes(srv, p, buf + n - p);
+               wrBytes(SrvSock, buf2, sprintf(buf2, "X-Pil: *Cipher=%s\r\n", SSL_get_cipher(ssl)));
+            wrBytes(SrvSock, p, buf + n - p);
 
             iSignal(SIGALRM, doSigAlarm);
             iSignal(SIGUSR1, doSigUsr1);
             if (Buddy = fork()) {
                for (;;) {
                   alarm(420);
-                  n = slow(ssl, cli, buf, sizeof(buf));
+                  n = slow(ssl, CliSock, buf, sizeof(buf));
                   alarm(0);
                   if (!n)
                      break;
-                  wrBytes(srv, buf, n);
+                  wrBytes(SrvSock, buf, n);
                }
-               shutdown(cli, SHUT_RD);
-               shutdown(srv, SHUT_WR);
+               shutdown(CliSock, SHUT_RD);
+               shutdown(SrvSock, SHUT_WR);
             }
             else {
                Buddy = getppid();
-               while ((n = read(srv, buf, sizeof(buf))) > 0) {
+               while ((n = read(SrvSock, buf, sizeof(buf))) > 0) {
                   kill(Buddy, SIGUSR1);
                   alarm(420);
                   if (ssl)
                      sslWrite(ssl, buf, n);
                   else
-                     wrBytes(cli, buf, n);
+                     wrBytes(CliSock, buf, n);
                   alarm(0);
                }
-               shutdown(srv, SHUT_RD);
-               shutdown(cli, SHUT_WR);
+               shutdown(SrvSock, SHUT_RD);
+               shutdown(CliSock, SHUT_WR);
             }
             return 0;
          }
-         close(cli);
+         close(CliSock);
       }
    }
 }
