@@ -1,14 +1,9 @@
-/* 26aug16abu
+/* 02sep17abu
  * (c) Software Lab. Alexander Burger
  */
 
 #include "pico.h"
 
-#ifdef __CYGWIN__
-#include <sys/file.h>
-#define fcntl(fd,cmd,fl) 0
-#endif
-
 static any read0(bool);
 
 // I/O Tokens
@@ -711,6 +706,7 @@ void rdOpen(any ex, any x, inFrame *f) {
          close(pfd[0]);
          if (pfd[1] != STDOUT_FILENO)
             dup2(pfd[1], STDOUT_FILENO),  close(pfd[1]);
+         signal(SIGPIPE, SIG_DFL);
          execvp(av[0], av);
          execError(av[0]);
       }
@@ -2160,6 +2156,11 @@ any doOut(any ex) {
    return x;
 }
 
+// (fd) -> cnt
+any doFd(any ex) {
+   return boxCnt(currFd(ex, (char*)&ex));
+}
+
 // (err 'sym . prg) -> any
 any doErr(any ex) {
    any x;
