@@ -1,4 +1,4 @@
-/* 24nov16abu
+/* 05apr17abu
  * (c) Software Lab. Alexander Burger
  */
 
@@ -778,34 +778,47 @@ any doCut(any ex) {
    return val(Pop(c1));
 }
 
-// (del 'any 'var) -> lst
+// (del 'any 'var ['flg]) -> lst
 any doDel(any ex) {
    any x, y;
+   bool flg;
    cell c1, c2, c3;
 
    x = cdr(ex),  Push(c1, EVAL(car(x)));
-   x = cdr(x),  Push(c2, EVAL(car(x)));
+   x = cadr(x),  Push(c2, EVAL(x));
+   flg = !isNil(EVAL(cadddr(ex)));
    NeedVar(ex,data(c2));
    CheckVar(ex,data(c2));
    if (isSym(data(c2)))
       Touch(ex,data(c2));
-   if (isCell(x = val(data(c2)))) {
-      if (equal(data(c1), car(x))) {
+   x = val(data(c2));
+   for (;;) {
+      if (!isCell(x)) {
          drop(c1);
-         return val(data(c2)) = cdr(x);
+         return x;
       }
-      Push(c3, y = cons(car(x), Nil));
-      while (isCell(x = cdr(x))) {
-         if (equal(data(c1), car(x))) {
-            cdr(y) = cdr(x);
-            drop(c1);
-            return val(data(c2)) = data(c3);
-         }
-         y = cdr(y) = cons(car(x), Nil);
+      if (!equal(data(c1), car(x)))
+         break;
+      val(data(c2)) = x = cdr(x);
+      if (!flg) {
+         drop(c1);
+         return x;
       }
    }
+   Push(c3, y = cons(car(x), Nil));
+   while (isCell(x = cdr(x))) {
+      if (equal(data(c1), car(x))) {
+         if (flg)
+            continue;
+         cdr(y) = cdr(x);
+         drop(c1);
+         return val(data(c2)) = data(c3);
+      }
+      y = cdr(y) = cons(car(x), Nil);
+   }
+   cdr(y) = x;
    drop(c1);
-   return val(data(c2));
+   return val(data(c2)) = data(c3);
 }
 
 // (queue 'var 'any) -> any
